package com.allinpay.yst2.demo.tx;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.security.PrivateKey;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.Header;
import org.apache.http.HeaderElement;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

import com.allinpay.yst2.demo.util.DemoSM2Utils;

/**
 * Created with IntelliJ IDEA.
 *
 * @author gejunqing
 * @version 1.0
 * @date 2024/2/8
 */
public class FileDownloadDemo extends TxDemo
{
    private static final String FILEPATH = "/Users/wuxiaoli/Documents/";//此处替换为本地文件夹地址

    public static void main(String[] args)
    {
        System.out.println(download("1", "20240228"));
    }

    private static String download(String fileType, String fileDate)
    {
        String result = "";
        try
        {
            // 创建HttpClient
            CloseableHttpClient httpClient = HttpClients.createDefault();
            HttpPost httpPost = new HttpPost(demoConfig.getFileDownloadUrl());
            List<NameValuePair> parameters = new ArrayList<NameValuePair>();
            NameValuePair appIdV = new BasicNameValuePair("appId", demoConfig.getAppId());
            NameValuePair timestampV = new BasicNameValuePair("fileType", fileType);
            NameValuePair tokenV = new BasicNameValuePair("fileDate", fileDate);
            parameters.add(appIdV);
            parameters.add(timestampV);
            parameters.add(tokenV);
            // 加签开始
            String srcMsg = demoConfig.getAppId() + fileType + fileDate;
            PrivateKey privateKey = DemoSM2Utils.privKeySM2FromBase64Str(demoConfig.getPrivateKeyStr());
            String sign = DemoSM2Utils.sign(privateKey, srcMsg);
            NameValuePair signV = new BasicNameValuePair("sign", sign);
            parameters.add(signV);
            // 加签结束
            UrlEncodedFormEntity formEntity = new UrlEncodedFormEntity(parameters);
            httpPost.setEntity(formEntity);
            // 执行提交
            CloseableHttpResponse response = httpClient.execute(httpPost);
            // 收到返回，写文件
            HttpEntity responseEntity = response.getEntity();
            if (response.getStatusLine().getStatusCode() == 200)
            {
                if (responseEntity != null)
                {
                    Header contentType = responseEntity.getContentType();
                    logger.info("contentType=" + contentType.getValue());
                    if ("text/html;charset=UTF-8".equals(contentType.getValue()))
                    {
                        result = "下载失败,响应结果:" + EntityUtils.toString(responseEntity);
                    }
                    else
                    {
                        InputStream is = responseEntity.getContent();
                        File file = new File(FILEPATH + getFileName(response));
                        file.getParentFile().mkdirs();
                        FileOutputStream fileout = new FileOutputStream(file);
                        /**
                         * 根据实际运行效果 设置缓冲区大小
                         */
                        byte[] buffer = new byte[1024];
                        int ch = 0;
                        while ((ch = is.read(buffer)) != -1)
                        {
                            fileout.write(buffer, 0, ch);
                        }
                        is.close();
                        fileout.flush();
                        fileout.close();

                        result = "文件下载成功,路径:" + file.getAbsolutePath();
                    }
                }
            }
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }

        return result;
    }

    /**
     * 获取response header中Content-Disposition中的filename值
     *
     * @param response
     * @return
     */
    public static String getFileName(CloseableHttpResponse response)
    {
        Header contentHeader = response.getFirstHeader("Content-Disposition");
        String filename = null;
        if (contentHeader != null)
        {
            HeaderElement[] values = contentHeader.getElements();
            if (values.length == 1)
            {
                NameValuePair param = values[0].getParameterByName("filename");
                if (param != null)
                {
                    try
                    {
                        filename = param.getValue();
                    }
                    catch (Exception e)
                    {
                        e.printStackTrace();
                    }
                }
            }
        }
        return filename;
    }
}
