package com.allinpay.yst2.demo.tx;

import java.io.File;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.security.MessageDigest;
import java.security.PrivateKey;
import java.util.Base64;
import java.util.Date;

import org.apache.commons.lang3.time.FastDateFormat;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

import com.allinpay.yst2.demo.util.DemoSM2Utils;

/**
 * Created with IntelliJ IDEA.
 *
 * @author gejunqing
 * @version 1.0
 * @date 2024/2/8
 */
public class FileUploadDemo extends TxDemo
{
    private static final String FILEPATH = "/Users/admin/Documents/test/yst2/test的副本.txt";

    public static void main(String[] args)
    {
        System.out.println(upload(FILEPATH, "0"));
    }

    private static String upload(String filePath, String fileType)
    {
        String result = "";
        try
        {
            String timestamp = FastDateFormat.getInstance("yyyy-MM-dd HH:mm:ss").format(new Date());
            File file = new File(filePath);
            InputStream is = Files.newInputStream(file.toPath());
            // 创建HttpClient
            CloseableHttpClient httpClient = HttpClients.createDefault();
            HttpPost httpPost = new HttpPost(demoConfig.getFileUploadUrl());

            MultipartEntityBuilder builder = MultipartEntityBuilder.create().setCharset(StandardCharsets.UTF_8);//设置请求的编码格式
            //此处一定要用RFC6532，网上普遍用的BROWSER_COMPATIBLE依然会出现中文名乱码
            builder.setMode(HttpMultipartMode.RFC6532);
            /* 绑定文件参数，传入文件流和contenttype，此处也可以继续添加其他formdata参数 */
            builder.addBinaryBody("file", is, ContentType.MULTIPART_FORM_DATA, file.getName());
            builder.addTextBody("appId", demoConfig.getAppId());
            builder.addTextBody("fileType", fileType);
            builder.addTextBody("timestamp", timestamp);
            // 计算MD5开始
            InputStream is1 = Files.newInputStream(file.toPath());
            String md5 = getFileMD5(is1);
            builder.addTextBody("md5", md5);
            // 计算MD5结束
            // 加签开始
            String srcMsg = demoConfig.getAppId() + fileType + md5 + timestamp;
            PrivateKey privateKey = DemoSM2Utils.privKeySM2FromBase64Str(demoConfig.getPrivateKeyStr());
            String sign = DemoSM2Utils.sign(privateKey, srcMsg);
            builder.addTextBody("sign", sign);
            // 加签结束
            HttpEntity entity = builder.build();
            httpPost.setEntity(entity);
            // 执行提交
            HttpResponse response = httpClient.execute(httpPost);
            HttpEntity responseEntity = response.getEntity();
            if (responseEntity != null)
            {
                // 将响应的内容转换成字符串
                result = EntityUtils.toString(responseEntity, StandardCharsets.UTF_8);
            }
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }

        return result;
    }

    public static String getFileMD5(InputStream in)
    {
        if (in == null)
        {
            return null;
        }
        MessageDigest digest = null;
        byte buffer[] = new byte[1024];
        int len;
        try
        {
            digest = MessageDigest.getInstance("MD5");
            while ((len = in.read(buffer, 0, 1024)) != -1)
            {
                digest.update(buffer, 0, len);
            }
            in.close();
        }
        catch (Exception e)
        {
            e.printStackTrace();
            return null;
        }
        byte[] bytes = Base64.getEncoder().encode(digest.digest());
        String callback = new String(bytes);
        return callback;
    }
}
